@@ -305,7 +305,7 @@ with some simple extensions.
       (nconc (cadr top) (list nil)))
     (setcar (nthcdr 3 (cadr top)) parameters)
     (gnus-dribble-enter
-     (format "(gnus-topic-set-parameters %s '%S)" topic parameters))))
+     (format "(gnus-topic-set-parameters %S '%S)" topic parameters))))
 
 (defun gnus-group-topic-parameters (group)
   "Compute the group parameters for GROUP taking into account inheritance from topics."
@@ -371,17 +371,18 @@ If LOWEST is non-nil, list all newsgroups of level LOWEST or higher."
        regexp))
 
     ;; Use topics.
-    (when (< lowest gnus-level-zombie)
-      (if list-topic
-	  (let ((top (gnus-topic-find-topology list-topic)))
-	    (gnus-topic-prepare-topic (cdr top) (car top)
-				      (or topic-level level) all))
-	(gnus-topic-prepare-topic gnus-topic-topology 0
-				  (or topic-level level) all))))
-
-  (gnus-group-set-mode-line)
-  (setq gnus-group-list-mode (cons level all))
-  (run-hooks 'gnus-group-prepare-hook))
+    (prog1
+	(when (< lowest gnus-level-zombie)
+	  (if list-topic
+	      (let ((top (gnus-topic-find-topology list-topic)))
+		(gnus-topic-prepare-topic (cdr top) (car top)
+					  (or topic-level level) all))
+	    (gnus-topic-prepare-topic gnus-topic-topology 0
+				      (or topic-level level) all)))
+      
+      (gnus-group-set-mode-line)
+      (setq gnus-group-list-mode (cons level all))
+      (run-hooks 'gnus-group-prepare-hook))))
 
 (defun gnus-topic-prepare-topic (topicl level &optional list-level all silent)
   "Insert TOPIC into the group buffer.
@@ -641,7 +642,7 @@ articles in the topic and its subtopics."
   (let* ((tgroups (apply 'append (mapcar (lambda (entry) (cdr entry))
 					 gnus-topic-alist)))
 	 (entry (assoc (caar gnus-topic-topology) gnus-topic-alist))
-	 (newsrc gnus-newsrc-alist)
+	 (newsrc (cdr gnus-newsrc-alist))
 	 group)
     (while newsrc
       (unless (member (setq group (gnus-info-group (pop newsrc))) tgroups)
@@ -743,8 +744,10 @@ articles in the topic and its subtopics."
 
 (defun gnus-topic-goto-next-group (group props)
   "Go to group or the next group after group."
-  (if (null group)
-      (gnus-topic-goto-topic (symbol-name (cadr (memq 'gnus-topic props))))
+  (if (not group)
+      (if (not (memq 'gnus-topic props))
+	  (goto-char (point-max))
+	(gnus-topic-goto-topic (symbol-name (cadr (memq 'gnus-topic props)))))
     (if (gnus-group-goto-group group)
 	t
       ;; The group is no longer visible.
@@ -761,9 +764,11 @@ articles in the topic and its subtopics."
 		      (not (gnus-group-goto-group (car after))))
 	    (setq after (cdr after))))
 	;; Finally, just put point on the topic.
-	(unless after
-	  (gnus-topic-goto-topic (car list))
-	  (setq after nil))
+	(if (not (car list))
+	    (goto-char (point-min))
+	  (unless after
+	    (gnus-topic-goto-topic (car list))
+	    (setq after nil)))
 	t))))
 
 ;;; Topic-active functions
@@ -1129,7 +1134,7 @@ If COPYP, copy the groups instead."
 
 (defun gnus-topic-mark-topic (topic &optional unmark)
   "Mark all groups in the topic with the process mark."
-  (interactive (list (gnus-current-topic)))
+  (interactive (list (gnus-group-topic-name)))
   (if (not topic)
       (call-interactively 'gnus-group-mark-group)
     (save-excursion
@@ -1140,9 +1145,9 @@ If COPYP, copy the groups instead."
 
 (defun gnus-topic-unmark-topic (topic &optional unmark)
   "Remove the process mark from all groups in the topic."
-  (interactive (list (gnus-current-topic)))
+  (interactive (list (gnus-group-topic-name)))
   (if (not topic)
-      (call-interactively 'gnus-group-mark-group)
+      (call-interactively 'gnus-group-unmark-group)
     (gnus-topic-mark-topic topic t)))
 
 (defun gnus-topic-get-new-news-this-topic (&optional n)
@@ -1282,6 +1287,7 @@ If performed on a topic, edit the topic parameters instead."
       ;; !!!Sometimes nil elements sneak into the alist,
       ;; for some reason or other.
       (setcar alist (delq nil (car alist)))
+      (setcar alist (delete "dummy.group" (car alist)))
       (gnus-topic-sort-topic (pop alist) func reverse))))
 
 (defun gnus-topic-sort-topic (topic func reverse)
