@@ -188,7 +188,8 @@ all.  This may very well take some time.")
      (t
       (nnheader-report 'nnml "Article %s retrieved" id)
       ;; We return the article number.
-      (cons group (string-to-int (file-name-nondirectory path)))))))
+      (cons (if group-num (car group-num) group)
+	    (string-to-int (file-name-nondirectory path)))))))
 
 (deffoo nnml-request-group (group &optional server dont-check)
   (cond 
@@ -710,11 +711,8 @@ all.  This may very well take some time.")
 		   (file-directory-p dir))
 	  (nnml-generate-nov-databases-1 dir seen))))
     ;; Do this directory.
-    (let ((files (sort
-		  (mapcar
-		   (lambda (name) (string-to-int name))
-		   (directory-files dir nil "^[0-9]+$" t))
-		  '<)))
+    (let ((files (sort (nnheader-article-to-file-alist dir)
+		       (lambda (a b) (< (car a) (car b))))))
       (when files
 	(funcall nnml-generate-active-function dir)
 	;; Generate the nov file.
@@ -728,10 +726,10 @@ all.  This may very well take some time.")
     (setq nnml-group-alist
 	  (delq (assoc group nnml-group-alist) nnml-group-alist))
     (push (list group
-		(cons (car files)
+		(cons (caar files)
 		      (let ((f files))
 			(while (cdr f) (setq f (cdr f)))
-			(car f))))
+			(caar f))))
 	  nnml-group-alist)))
 
 (defun nnml-generate-nov-file (dir files)
@@ -749,8 +747,7 @@ all.  This may very well take some time.")
       (when (file-exists-p nov)
 	(funcall nnmail-delete-file-function nov))
       (while files
-	(unless (file-directory-p 
-		 (setq file (concat dir (int-to-string (car files)))))
+	(unless (file-directory-p (setq file (concat dir (cdar files))))
 	  (erase-buffer)
 	  (nnheader-insert-file-contents file)
 	  (narrow-to-region 
@@ -762,7 +759,7 @@ all.  This may very well take some time.")
 	  (when (and (not (= 0 chars))	; none of them empty files...
 		     (not (= (point-min) (point-max))))
 	    (goto-char (point-min))
-	    (setq headers (nnml-parse-head chars (car files)))
+	    (setq headers (nnml-parse-head chars (caar files)))
 	    (save-excursion
 	      (set-buffer nov-buffer)
 	      (goto-char (point-max))
